from __future__ import unicode_literals

from django import forms
from django.contrib.auth.models import User
from django.contrib.auth.forms import UserCreationForm
from inviter2.models import OptOut


class RegistrationForm(UserCreationForm):
    """Slightly modified version of the standard Django UserCreationForm to
    enable the `is_active` field from the default of False."""

    class Meta:
        model = User
        fields = ('username', 'email')

    def __init__(self, *args, **kwargs):
        super(RegistrationForm, self).__init__(*args, **kwargs)
        # Hide the UUID string
        self.initial['username'] = ''

    def save(self, *args, **kwargs):
        user = super(RegistrationForm, self).save(*args, **kwargs)
        user.is_active = True
        if kwargs.get('commit', True):
            user.save()
        return user


class OptOutForm(forms.ModelForm):
    """ Dummy form for opting out. """
    class Meta:
        model = User
        fields = ()

    def save(self):
        """ Delete the user object from the database and store the SHA1 hashed
        email address in the database to make sure this person does not receive
        any further invitation emails. """

        email = self.instance.email
        self.instance.delete()
        return OptOut.objects.create(email=email)



#!/usr/bin/env python

from setuptools import setup, find_packages
import inviter2

setup(
    name='django-inviter2',
    version=inviter2.__version__,
    description='Simple email invitations for your Django app',
    long_description=open('README.rst').read(),
    author='Michael J Schultz',
    author_email='mjschultz@gmail.com',
    url='https://github.com/mjschultz/django-inviter2',
    packages=find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    install_requires=[
        "shortuuid >= 0.1",
        "six",
        "Django >= 1.4",
    ],
    include_package_data=True,
    zip_safe=False,
)



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
from __future__ import unicode_literals

from shortuuid import uuid

from six.moves.urllib.parse import urlparse

from django.contrib.auth import get_user_model
from django.core import mail
from django.core.urlresolvers import reverse
from django.http import Http404
from django.test import TestCase
from django.utils.http import int_to_base36

from .models import OptOut
from .utils import invite, token_generator
from .views import UserMixin


User = get_user_model()


class InviteTest(TestCase):
    def setUp(self):
        self.required_fields = getattr(User, 'REQUIRED_FIELDS', [])
        self.required_fields.append(
            getattr(User, 'USERNAME_FIELD', 'username'))
        if 'username' in self.required_fields:
            self.inviter = User.objects.create(username=uuid())
            self.existing = User.objects.create(
                email='existing@example.com', username=uuid())
        else:
            self.inviter = User.objects.create()
            self.existing = User.objects.create(email='existing@example.com')

    def test_inviting(self):
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        # Resend the mail
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        # Don't resend the mail
        user, sent = invite("foo@example.com", self.inviter, resend=False)
        self.assertFalse(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        # Don't send the email to active users
        user, sent = invite("existing@example.com", self.inviter)
        self.assertFalse(sent)
        self.assertTrue(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

    def test_reinvite_user(self):
        self.existing.is_active = False
        self.existing.save()

        user, sent = invite("existing@example.com", self.inviter,
                            user=self.existing)
        self.assertTrue(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(2, User.objects.count())

    def test_views(self):
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        url_parts = int_to_base36(user.id), token_generator.make_token(user)

        url = reverse('inviter2:register', args=url_parts)

        resp = self.client.get(url)

        self.assertEqual(200, resp.status_code, resp.status_code)
        
        fields = {
            'email': 'foo@example.com',
            'password1': 'test-1234',
            'password2': 'test-1234'
        }
        if 'username' in self.required_fields:
            fields.update(username='testuser')
        
        resp = self.client.post(url, fields)

        self.assertEqual(302, resp.status_code, resp.content)

        # self.client.login(username='testuser', password='test-1234')

        resp = self.client.get(reverse('inviter2:done'))

        self.assertEqual(200, resp.status_code, resp.status_code)

    def test_error_views(self):
        # invalid base36 encoded user id
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        url_parts = 'z' * 13, token_generator.make_token(user)
        url = reverse('inviter2:register', args=url_parts)
        resp = self.client.get(url)
        self.assertEqual(404, resp.status_code, resp.status_code)

        # attempt to use some other user's id
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        uid = int_to_base36(self.existing.id)
        token = token_generator.make_token(user)
        url = reverse('inviter2:register', args=(uid, token))
        resp = self.client.get(url)
        self.assertEqual(403, resp.status_code, resp.status_code)

        # submit an invalid form
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        url_parts = int_to_base36(user.id), token_generator.make_token(user)
        url = reverse('inviter2:register', args=url_parts)
        resp = self.client.get(url)
        self.assertEqual(200, resp.status_code, resp.status_code)
        
        fields = {
            'email': 'foo@example.com',
            'password1': 'test-1234',
            'password2': 'test-4321'
        }
        if 'username' in self.required_fields:
            fields.update(username='testuser')
        
        resp = self.client.post(url, fields)
        self.assertEqual(200, resp.status_code, resp.content)
        self.assertIn(
            'The two password fields didn&#39;t match.', str(resp.content))

        # developer with bad redirect URL
        with self.settings(INVITER_REDIRECT='http://example.com/'):
            fields.update(password2='test-1234')
            resp = self.client.post(url, fields)
            self.assertEqual(302, resp.status_code, resp.content)
            self.assertEqual(resp['Location'], 'http://example.com/')

    def test_get_user(self):
        mixin = UserMixin()
        with self.assertRaises(Http404) as e:
            mixin.get_user('z' * 14)
        self.assertEqual(str(e.exception), 'No such invited user.')

    def test_opt_out(self):
        self.assertEqual(2, User.objects.count())

        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        url_parts = int_to_base36(user.id), token_generator.make_token(user)
        url = reverse('inviter2:opt-out', args=url_parts)

        resp = self.client.get(url)
        self.assertEqual(200, resp.status_code, resp.status_code)

        resp = self.client.post(url, {})
        self.assertEqual(302, resp.status_code, resp.status_code)
        self.assertEqual(reverse('inviter2:opt-out-done'),
                         urlparse(resp['Location']).path)
        self.assertEqual(2, User.objects.count())

        user, sent = invite("foo@example.com", self.inviter)
        self.assertFalse(sent)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(1, OptOut.objects.count())
        self.assertTrue(OptOut.objects.is_blocked("foo@example.com"))
        self.assertIsNone(user)

        opt_out = OptOut.objects.get()
        opt_hash = OptOut.objects._hash_email('foo@example.com')
        self.assertEqual(opt_out.__unicode__(), opt_hash)

    def test_opt_out_done(self):
        resp = self.client.get(reverse('inviter2:opt-out-done'))
        self.assertEqual(200, resp.status_code, resp.status_code)



from __future__ import unicode_literals

from django.db import models
import hashlib


class OptOutManager(models.Manager):
    def _hash_email(self, email):
        email_bytes = email.encode('utf-8')
        return hashlib.sha1(email_bytes).hexdigest()

    def is_blocked(self, email=None):
        """ Check if a given email address is on the block list. """
        return self.filter(hash=self._hash_email(email)).count() > 0

    def create(self, email=None):
        """ Create an opt out. """
        return super(OptOutManager, self).create(hash=self._hash_email(email))


class OptOut(models.Model):
    """ Opt-out email addresses are stored as SHA1 hashes to make sure we don't
    accidentally collect any more data once a person signalled they're not
    interested in receiving any more invitation emails from us. """
    hash = models.CharField(max_length=255)

    objects = OptOutManager()

    def __unicode__(self):
        return self.hash



